# coding=utf-8
# pylint: disable=missing-docstring,redefined-outer-name

import pytest

from xls_writer import detail, exceptions
from xls_writer.field import FieldFactory

from . import helpers


@pytest.fixture()
def field_reader():
  return detail.DefaultFieldReader("foo.bar.baz")


def test_path(field_reader):
  assert field_reader.path == ["foo", "bar", "baz"]


@pytest.mark.parametrize("instance", helpers.foobarbaz("canary"))
def test_extraction(field_reader, instance):
  assert field_reader(field=None, instance=instance) == "canary"


def test_missing(field_reader):
  with pytest.raises(exceptions.MissingField):
    field_reader(None, object())


@pytest.mark.parametrize("instance", helpers.foobarbaz("canary"))
def test_const_field_reader(instance):
  canary = object()
  field_reader = detail.ConstReader(canary)
  assert field_reader(field=None, instance=instance) == canary


def test_multi_reader():
  instance = detail.MultiReader({
    "foo": FieldFactory.create_const_field("Foo", "foo"),
    "bar": FieldFactory.create_const_field("Bar", "bar"),
  })
  assert instance(None, None) == {'bar': 'bar', 'foo': 'foo'}



# coding=utf-8
"""Misc utils"""

import typing

from . import api


def store_table(
    row_source: typing.Sequence[object],
    table: api.TableDescription,
    formatter_factory: api.FormatterFactory,
    output_file,
    close_file_on_finish: bool = True
):
  """Store table using formatter."""
  formatter = formatter_factory.create()
  try:
    formatter.initialize_file(output_file)
    formatter.write_header(table.header)
    for row in row_source:
      formatter.write_row(table.extract_row(row))
  finally:
    formatter.close(close_file_on_finish)





# coding=utf-8
# pylint: disable=missing-docstring

import pytest

from xls_writer import field, detail


def test_header():
  tested_field = field.FieldFactory("a header")
  assert tested_field.header == "a header"


def test_required_default():
  tested_field = field.FieldFactory("a header")
  assert not tested_field.required


@pytest.mark.parametrize('required_state', [True, False])
def test_required(required_state):
  tested_field = field.FieldFactory("a header", required=required_state)
  assert tested_field.required == required_state


def test_default_default():
  tested_field = field.FieldFactory("a header")
  assert tested_field.default is field._NO_DEFAULT  # pylint: disable=protected-access


@pytest.mark.parametrize('default', list(range(10)))
def test_default_value(default):
  tested_field = field.FieldFactory("a header", default=default).create()
  assert tested_field.default is default


def test_field_reader():
  canary = object()
  tested_field = field.FieldFactory("a header").reader(canary).create()
  assert tested_field.field_reader is canary


def test_path_reader():
  path_canary = "foo.bar.baz"
  tested_field = field.FieldFactory("a header").path(path_canary).create()
  assert isinstance(tested_field.field_reader, detail.DefaultFieldReader)
  assert tested_field.field_reader.path == ['foo', 'bar', 'baz']


def test_const_reader():
  const_canary = object()
  tested_field = field.FieldFactory("a header").const_field(const_canary).create()
  assert isinstance(tested_field.field_reader, detail.ConstReader)
  assert tested_field.field_reader.instance == const_canary


@pytest.mark.parametrize('coalesce_to', [int, float, str])
def test_type_formatter(coalesce_to):
  tested_field = field.FieldFactory("a header", required=True) \
    .path("foo.bar.baz").type_formatter(coalesce_to).create()
  assert isinstance(tested_field.field_formatter, detail.TypeFormatter)
  assert tested_field.field_formatter.object_type == coalesce_to


def test_field_formatter():
  formatter_canary = object()
  tested_field = field.FieldFactory("a header", required=True) \
    .path("foo.bar.baz").formatter(formatter_canary).create()

  assert tested_field.field_formatter is formatter_canary


def test_empty_check_default():
  tested_field = field.FieldFactory("a header").create()
  assert isinstance(tested_field.field_empty_check, detail.DefaultFieldEmptyCheck)


def test_empty_check_noop():
  tested_field = field.FieldFactory("a header").noop_empty_check().create()
  assert isinstance(tested_field.field_empty_check, detail.NoopEmptyCheck)


def test_empty_check():
  empty_check_canary = object()
  tested_field = field.FieldFactory("a header").empty_check(empty_check_canary).create()
  assert tested_field.field_empty_check is empty_check_canary


def test_enumerated_empty_check():
  canary = {1, 2, 3}
  tested_field = field.FieldFactory("a header").enumerated_empty_check(canary).create()
  assert isinstance(tested_field.field_empty_check, detail.EnumeratedFieldEmptyCheck)
  assert tested_field.field_empty_check.empty_values == canary



# coding=utf-8
# pylint: disable=missing-docstring

import pytest

from xls_writer import detail


default_test_data = (
  (None, True),
  (0, False),
  ("", False),
  ("non empty", False),
  (1, False)
)


@pytest.mark.parametrize("value, expected", default_test_data)
def test_default_empty_check(value, expected):
  check = detail.DefaultFieldEmptyCheck()
  assert expected == check(field=None, instance=value)


enumerated_empty_check_test_data = (
  # When no values are considered empty all values are not empty
  ([], None, False),
  ([], 0, False),
  ([], "", False),

  # Only values marked as empty are considered empty
  ([0, -1, None], None, True),
  ([0, -1, None], -1, True),
  ([0, -1, None], 0, True),
  ([0, -1, None], "", False),
  ([0, -1, None], "non empty", False),
  ([0, -1, None], 1, False),

  # Test for different containers for empty values
  ((0, -1, None), None, True),
  ({0, -1, None}, None, True),
  (iter({0, -1, None}), None, True),
)


@pytest.mark.parametrize("empty, value, expected", enumerated_empty_check_test_data)
def test_enumerated_check(empty, value, expected):
  check = detail.EnumeratedFieldEmptyCheck(empty_values=empty)
  assert expected == check(field=None, instance=value)



# coding=utf-8
# pylint: disable=missing-docstring,redefined-outer-name

import pytest

from xls_writer import field, exceptions

from . import helpers


int_canaries = [0, 1, 2]
float_canaries = [0.0, 1.0, 2.0]
str_canaries = ["2", "1", "0"]


default_canaries = int_canaries + float_canaries + str_canaries


@pytest.fixture(params=default_canaries)
def canary(request):
  return request.param


@pytest.fixture()
def foobarbaz_with_canary(canary):
  return helpers.foobarbaz(canary)


def test_simple_field(canary, foobarbaz_with_canary):
  tested_field = field.FieldFactory("a header").path("foo.bar.baz").create()
  for instance in foobarbaz_with_canary:
    assert tested_field(row=instance) == canary


def test_simple_field_none():
  tested_field = field.FieldFactory("a header").path("foo.bar.baz").create()
  for instance in helpers.foobarbaz(None):
    assert tested_field(row=instance) is None


def test_simple_default():
  tested_field = field.FieldFactory("a header", default="canary").path("foo.bar.baz").create()
  for instance in helpers.foobarbaz(None):
    assert tested_field(row=instance) == "canary"


def test_missing_default():
  tested_field = field.FieldFactory("a header", default="canary", required=False) \
    .path("foo.bar.baz").create()

  assert tested_field(row=object) == "canary"


def test_simple_raises():
  tested_field = field.FieldFactory("a header", required=True).path("foo.bar.baz").create()
  for instance in helpers.foobarbaz(None):
    with pytest.raises(exceptions.MissingField):
      tested_field(row=instance)


@pytest.mark.parametrize('coalesce_to', [int, float, str])
def test_field_type_formatter(coalesce_to, canary, foobarbaz_with_canary):
  tested_field = field.FieldFactory("a header", required=True) \
    .path("foo.bar.baz").type_formatter(coalesce_to).create()
  for instance in foobarbaz_with_canary:
    assert tested_field(row=instance) == coalesce_to(canary)


def test_format_formatter(canary, foobarbaz_with_canary):
  tested_field = field.FieldFactory("a header").path(
    "foo.bar.baz").format_formatter("foo {}").create()
  for instance in foobarbaz_with_canary:
    assert tested_field(row=instance) == "foo {}".format(canary)




# coding=utf-8
"""Module with all exceptions."""


class XLSWriter(Exception):
  """Base exception"""


class MissingField(XLSWriter):
  """Raised when we are missing required field with no default."""



# coding=utf-8

from setuptools import find_packages, setup
import pathlib
import os

MAIN_DIR = pathlib.Path(__file__).absolute().parent


# Did I mention that setup.py is not finest piece of software on earth.
# For this to work when installed you'll need to enumerate all template and static file.


def read_dir(package: str, directory: str):
  package_root = os.path.abspath(package.replace(".", "/"))
  directory = os.path.join(package_root, directory)
  res = []
  for root, subFolders, files in os.walk(directory):
    for file in files:
      res.append(
        os.path.relpath(
         os.path.join(root, file),
         package_root
        ))

  return res


packages = find_packages(
  str(MAIN_DIR),
  include=('xls_writer*',),
  exclude=[]
)


if __name__ == "__main__":

  setup(
    name='xls_writer',
    version='0.2.1',
    packages=packages,
    license='All Rights reserved',
    author='Jacek Bzdak',
    author_email='jacek@askesis.pl',
    description='A simple utility to format tabular data from python objects.',
    install_requires=[],
    package_data={
      package: [] +
        read_dir(package, "static") +
        read_dir(package, "templates")
      for package in packages
    },
    include_package_data=True
  )
